#!/usr/bin/env python3
# -*- coding: utf-8 -*-

###################
#    This file found strings in binary file.
#    Copyright (C) 2021  Maurice Lambert

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
###################

""" This file found strings in binary file.

>>> with open("test", 'wb') as f: f.write(b"\\x00\\x01abcde\\x00\\x01")
9
>>> strings = Strings("test")
>>> for line in strings.reader(): print(line)
abcde
>>> import os; os.remove("test")
"""

from typing import Generator
from argparse import ArgumentParser


class Strings:

    """This class implement a Strings getter from binary file.

    >>> with open("test", 'wb') as f: f.write(b"abc\\x00\\x01ab\\x00cdeF\\x00\\x01")
    14
    >>> strings = Strings("test", chars=b"abcde", minimum_chars=3, number_bad_chars=1, accepted_bad_chars=b"\\x00")
    >>> for line in strings.reader(): print(line)
    abc
    abcde
    >>> import os; os.remove("test")
    """

    def __init__(
        self,
        filename: str,
        chars: bytes = (
            b"0123456789"
            b"abcdefghijklmnopqrstuvwxyz"
            b"ABCDEFGHIJKLMNOPQRSTUVWXYZ"
            b"!\"\#$%&'()*+,-./:;<=>?@[\\]^_`{|}~ "
        ),
        minimum_chars: int = 5,
        number_bad_chars: int = 0,
        accepted_bad_chars: bytes = b"",
        encoding: str = "latin1",
    ):
        self.chars = chars
        self.minimum_chars = minimum_chars
        self.filename = filename
        self.number_bad_chars = number_bad_chars
        self.accepted_bad_chars = accepted_bad_chars
        self.current_string: str = ""
        self.current_bad_chars: int = 0
        self.encoding = encoding

    def reader(self) -> Generator[str, None, None]:

        """This function read file 1 chars by 1 chars and yield lines.

        >>> with open("test", 'wb') as f: f.write(b"abc\\x00\\x01ab\\x00cdeF\\x00\\x01")
        14
        >>> strings = Strings("test", chars=b"abcde", minimum_chars=3, number_bad_chars=1, accepted_bad_chars=b"\\x00")
        >>> for line in strings.reader(): print(line)
        abc
        abcde
        >>> import os; os.remove("test")
        """

        char = b" "

        with open(self.filename, "rb") as file:
            while char:
                char = file.read(1)

                if char:
                    string = self.analyse_char(char)
                    if string:
                        yield string

        if len(self.current_string) >= self.minimum_chars:
            yield self.current_string

    def analyse_char(self, char: bytes) -> str:

        """This function analyse a byte and return strings found.

        >>> strings = Strings(None)
        >>> [strings.analyse_char(x) for x in (b"a", b"b", b"c", b"d", b"e", b"\\x00")]
        [None, None, None, None, None, 'abcde']
        >>> strings = Strings(None, number_bad_chars=1)
        >>> [strings.analyse_char(x) for x in (b"a", b"b", b"\\x00", b"c", b"d", b"e", b"\\x00", b"\\x00")]
        [None, None, None, None, None, None, None, 'abcde']
        >>> strings = Strings(None, number_bad_chars=1, accepted_bad_chars=b"\\x00")
        >>> [strings.analyse_char(x) for x in (b"a", b"b", b"\\x00", b"c", b"d", b"e", b"\\x00", b"\\x00")]
        [None, None, None, None, None, None, None, 'abcde']
        >>> strings = Strings(None)
        >>> [strings.analyse_char(x) for x in (b"a", b"b", b"\\x00", b"c", b"d", b"e", b"\\x00", b"\\x00")]
        [None, None, None, None, None, None, None, None]
        >>> strings = Strings(None, number_bad_chars=1, accepted_bad_chars=b"\\x01")
        >>> [strings.analyse_char(x) for x in (b"a", b"b", b"\\x00", b"c", b"d", b"e", b"\\x00", b"\\x00")]
        [None, None, None, None, None, None, None, None]
        """

        if char in self.chars:
            self.current_string += char.decode(self.encoding)
            self.current_bad_chars = 0
            return

        elif (self.accepted_bad_chars and char in self.accepted_bad_chars) or (
            not self.accepted_bad_chars
        ):
            self.current_bad_chars += 1
            string = self.current_string

            if (
                self.current_bad_chars > self.number_bad_chars
                and len(string) >= self.minimum_chars
            ):
                self.current_string = ""
                return string

            elif self.current_bad_chars > self.number_bad_chars:
                self.current_string = ""
                return

        else:
            string = self.current_string
            self.current_bad_chars = 0
            self.current_string = ""

            if len(string) >= self.minimum_chars:
                return string
            return


def main() -> None:

    """This function parse arguments and return error."""

    parser = ArgumentParser()
    parser.add_argument("filename", help="Filename of binary file.")
    parser.add_argument(
        "--min-chars",
        "-m",
        type=int,
        default=5,
        help="Minimum length to print a string.",
    )
    parser.add_argument(
        "--max-bad-chars",
        "-M",
        type=int,
        default=0,
        help="Maximum bad characters in string.",
    )
    args = parser.parse_args()

    strings = Strings(
        args.filename, minimum_chars=args.min_chars, number_bad_chars=args.max_bad_chars
    )
    for line in strings.reader():
        print(line)
    exit(0)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

""" This package read binary file to get all strings or read it with hexareader. """

###################
#    This package read binary file to get all strings or read it with hexareader.
#    Copyright (C) 2021  Maurice Lambert

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
###################

print(
"""
BinaryFileReader  Copyright (C) 2021  Maurice Lambert
This program comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it
under certain conditions.
"""
)

try:
    from .Strings import Strings, main as get_strings
    from .HexaReader import HexaReader, main as hexaread
except ImportError:
    from Strings import Strings, main as get_strings
    from HexaReader import HexaReader, main as hexaread

from sys import argv

help_message = """USAGE: python3 -m BinaryFileReader module filename
    [module] must be "strings" or "hexareader"
    [filename] must be an existing binary file
"""

if len(argv) > 1:
    module = argv.pop(1).lower()

    if module == "strings":
        get_strings()
        exit(0)
    elif module == "hexareader":
        hexaread()
        exit(0)
    else:
        print(help_message)
        exit(1)
else:
    print(help_message)
    exit(1)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

###################
#    This file implement a HexaReader (read binary file as hexa and ascii).
#    Copyright (C) 2021  Maurice Lambert

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
###################

""" This file implement a HexaReader (read binary file as hexa and ascii).

>>> with open("test", 'wb') as f: f.write(b"\\x00abc")
4
>>> hexareader = HexaReader("test")
>>> for line in hexareader.reader(): print(line)
00 61 62 63                                  .abc
>>> import os; os.remove("test")
"""

from typing import Generator
from binascii import hexlify
from os import path
from sys import argv


class HexaReader:

    """This class implement a hexadecimal reader for binary file.

    >>> with open("test", 'wb') as f: f.write(b"\\x00abc")
    4
    >>> hexareader = HexaReader("test")
    >>> for line in hexareader.reader(): print(line)
    00 61 62 63                                  .abc
    >>> import os; os.remove("test")
    """

    def __init__(self, filename: str):
        self.filename = filename
        self.chars = (
            "0123456789"
            "abcdefghijklmnopqrstuvwxyz"
            "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
            "!\"\#$%&'()*+,-./:;<=>?@[\\]^_`{|}~ "
        ).encode()

    def reader(self) -> Generator[str, None, None]:

        """This function read file 16 chars by 16 chars and yield lines.

        >>> with open("test", 'wb') as f: f.write(b"\\x00abc")
        4
        >>> hexareader = HexaReader("test")
        >>> for line in hexareader.reader(): print(line)
        00 61 62 63                                  .abc
        >>> import os; os.remove("test")
        """

        data = b" "

        with open(self.filename, "rb") as file:
            while data:
                data = file.read(16)

                if data:
                    yield self.get_line(data)

    def get_line(self, data: bytes) -> str:

        """This function return hexareader line from bytes.

        >>> hexareader = HexaReader(None)
        >>> hexareader.get_line(b"\\x00\\x01")
        '00 01                                        ..'
        """

        hexa = "{:<55}".format(hexlify(data, " ").decode())

        ascii_ = ""
        for char in data:
            if char in self.chars:
                ascii_ += chr(char)
            else:
                ascii_ += "."

        return hexa + ascii_


def main() -> None:

    """This function parse arguments and return error."""

    if len(argv) != 2:
        print("USAGE: python3 HexaReader.py filename")
        exit(1)
    elif not path.exists(argv[1]):
        print(f"ERROR: file {argv[1]} doesn't exist.")
        exit(2)
    else:
        hexareader = HexaReader(argv[1])
        for line in hexareader.reader():
            print(line)
        exit(0)


if __name__ == "__main__":
    main()



from setuptools import setup, find_packages
import BinaryFileReader

setup(
    name = "BinaryFileReader",
 
    version = BinaryFileReader.__version__,
    packages = find_packages(),
    install_requires = [],

    author = "Maurice Lambert", 
    author_email = "mauricelambert434@gmail.com",
 
    description = "This package read binary file to get all strings or read it like a hexareader.",
    long_description = open('README.md').read(),
    long_description_content_type="text/markdown",
 
    include_package_data = True,

    url = 'https://github.com/mauricelambert/BinaryFileReader',
 
    classifiers = [
        "Programming Language :: Python",
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.9",
        "Operating System :: POSIX :: Linux",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: MacOS",
    ],
 
    entry_points = {
        'console_scripts': [
            'GetStrings = BinaryFileReader:get_strings',
            'HexaReader = BinaryFileReader:hexaread',
        ],
    },
    python_requires='>=3.6',
)